import math

import pyglet
from pyglet.gl import *

class Entity:
    """Represents an entity"""

    def __init__(self, x, y, z, player, game, texture):
        self.x = x
        self.y = y
        self.z = z
        self.player = player
        self.game = game
        self.texture = texture
    
    def draw(self):
        glEnable(GL_ALPHA_TEST);
        glAlphaFunc(GL_GREATER, 0.5);
        glEnable(GL_TEXTURE_2D)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        facing = math.degrees(math.atan2(self.x - self.player.x, self.y - self.player.y))
        glPushMatrix()
        glTranslatef(self.x, self.y, self.z)
        glRotatef(-facing, 0., 0., 1.)
        glBindTexture(self.texture.target, self.texture.id)
        glBegin(GL_QUADS)
        glTexCoord2i(0, 0)
        glVertex3f(-.4, 0., 0.)
        glTexCoord2i(1, 0)
        glVertex3f(.4, 0., 0.)
        glTexCoord2i(1, 1)
        glVertex3f(.4, 0., 1.5)
        glTexCoord2i(0, 1)
        glVertex3f(-.4, 0., 1.5)
        glEnd()
        glPopMatrix()
        glDisable(GL_BLEND)
    
    def pickup(self):
        return False
    
    def update(self, dt):
        pass

class Goal(Entity):
    """Represents the level goal"""
    
    texture = pyglet.resource.texture("tex/goal-closed.png")
    texture2 = pyglet.resource.texture("tex/goal-open.png")
    
    def __init__(self, x, y, player, game):
        Entity.__init__(self, x, y, 0, player, game, Goal.texture)
    
    def pickup(self):
        if self.player.stars == self.game.level.stars_max:
            self.game.nextLevel()
        else:
            self.game.restart()
        return True
    
    def update(self, dt):
        if self.player.stars == self.game.level.stars_max:
            self.texture = Goal.texture2


class Key(Entity):
    """Represents a key"""
    
    texture = pyglet.resource.texture("tex/key.png")
    
    def __init__(self, x, y, player, game):
        Entity.__init__(self, x, y, 0, player, game, Key.texture)
        self.bob = 1
        
    def pickup(self):
        self.player.keys += 1
        self.game.pickup.play()
        return True
    
    def update(self, dt):
        self.z += self.bob * dt
        if self.z < 0:
            self.z = 0
            self.bob = .1
        elif self.z > .1:
            self.z = .1
            self.bob = -.1

class Star(Entity):
    """Represents a star"""
    
    texture = pyglet.resource.texture("tex/star.png")
    
    def __init__(self, x, y, player, game):
        Entity.__init__(self, x, y, 0, player, game, Star.texture)
        self.bob = 1
        
    def pickup(self):
        self.player.stars += 1
        self.game.pickup.play()
        return True
    
    def update(self, dt):
        self.z += self.bob * dt
        if self.z < 0:
            self.z = 0
            self.bob = .1
        elif self.z > .1:
            self.z = .1
            self.bob = -.1
