import pyglet

from level import *
from player import *
from entity import *

class Game:
    """Kitchen sink class"""
    
    def __init__(self):
        self.level_num = 0
        self.level = None
        self.player = None
        self.entities = []
        
        self.blink = 0
        self.isNextLeveling = False
        self.isRestarting = False
        
        self.bgm = pyglet.resource.media('bgm/DST-Afternoon.mp3', streaming=True)
        self.bgm_player = pyglet.media.Player()
        self.bgm_player.queue(self.bgm)
        self.bgm_player.eos_action = pyglet.media.Player.EOS_LOOP
        self.bgm_player.volume = 0.4
        
        self.pickup = pyglet.resource.media('sfx/pickup.ogg', streaming=False)
        self.footsteps = pyglet.resource.media('sfx/footsteps.ogg', streaming=False)
        self.openDoor = pyglet.resource.media('sfx/key.ogg', streaming=False)

    def nextLevel(self):
        self.blink = 0
        self.isNextLeveling = True

    def do_nextLevel(self):
        self.level_num += 1
        self.level = Level()
        self.level.load_from_file("lvl/"+str(self.level_num)+".png")
        self.player = Player(self.level, self)
        self.reload_entities()
        
        # Play sound
        sound = pyglet.resource.media('sfx/'+str(self.level_num)+'.ogg',streaming=True)
        player = pyglet.media.Player()
        player.queue(sound)
        player.play()
    
    def reload_entities(self):
        del self.entities[:]
        for entity in self.level.entity_list:
            if entity[0] == "Goal":
                self.entities.append(Goal(entity[1], entity[2], self.player, self))
            if entity[0] == "Key":
                self.entities.append(Key(entity[1], entity[2], self.player, self))
            if entity[0] == "Star":
                self.entities.append(Star(entity[1], entity[2], self.player, self))

    def restart(self):
        self.blink = 0
        self.isRestarting = True
    
    def do_restart(self):
        self.level.reset()
        self.level.cache()
        self.player.respawn()
        self.reload_entities()
    
    def update(self, dt):
        if self.isNextLeveling:
            if self.blink >= 1:
                self.isNextLeveling = False
                self.do_nextLevel()
            else:
                self.blink += 2 * dt
        elif self.isRestarting:
            if self.blink >= 1:
                self.isRestarting = False
                self.do_restart()
            else:
                self.blink += dt
        else:
            if self.blink > 0:
                self.blink -= 2 * dt
            else:
                self.blink = 0
