import math

import pyglet
from pyglet.gl import *

pyglet.resource.path = ['res.zip', 'res']
pyglet.resource.reindex()

pyglet.resource.add_font('fnt/Audiowide-Regular.ttf')

from level import *
from player import *
from entity import *
from game import *

window = pyglet.window.Window(fullscreen=True)

isPlaying = False

title_image = pyglet.resource.texture('tex/title.png')
title_sprite = pyglet.sprite.Sprite(title_image,
        x=(window.width-title_image.width)/2,
        y=(window.height-title_image.height)/2)

stars_display = pyglet.text.Label('0',
        font_name='Audiowide',
        font_size=64,
        x=160, y=window.height-80)
stars_image = pyglet.resource.texture('tex/star.png')
stars_sprite = pyglet.sprite.Sprite(stars_image,
        x=0, y=window.height-96)
stars_sprite.scale = 196. / 512.

keys_display = pyglet.text.Label('0',
        font_name='Audiowide',
        font_size=64,
        x=160, y=window.height-192)
keys_image = pyglet.resource.texture('tex/key.png')
keys_sprite = pyglet.sprite.Sprite(keys_image,
        x=16, y=window.height-192)
keys_sprite.scale = 128. / 512.

joysticks = []
for joystick in pyglet.input.get_joysticks():
    joystick.open()
    joysticks.append(joystick)

glEnable(GL_CULL_FACE)
glEnable(GL_DEPTH_TEST)
glEnable(GL_TEXTURE_2D)

game = Game()
game.bgm_player.play()
game.do_nextLevel()

@window.event
def on_draw():
    window.clear()
    
    global isPlaying
    if not isPlaying:
        glViewport(0, 0, window.width, window.height)
        glMatrixMode(gl.GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0, window.width, 0, window.height, -1, 1)
        glMatrixMode(gl.GL_MODELVIEW)
        glLoadIdentity()
        title_sprite.draw()
        return

    # Draw 3D
    on_resize(window.width, window.height)
    glLoadIdentity()
    gluLookAt(game.player.x, game.player.y, .8, # eye
            game.player.x + math.cos(game.player.heading), game.player.y + math.sin(game.player.heading), .6, # centre
            0, 0, 1) # up
    
    game.level.draw()
    
    # Draw Entities
    if game.blink == 0:
	    glColor4f(1, 1, 1, 1)
	    for entity in game.entities:
	        entity.draw()
        
    # Holding
    if game.player.keys > 0:
        glDisable(GL_DEPTH_TEST)
        glEnable(GL_TEXTURE_2D)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glBindTexture(Key.texture.target, Key.texture.id)
        glPushMatrix()
        glTranslatef(game.player.x + math.cos(game.player.heading), game.player.y + math.sin(game.player.heading), 0)
        glRotatef(math.degrees(game.player.heading), 0., 0., 1.)
        glBegin(GL_QUADS)
        glTexCoord2i(0, 0)
        glVertex3f(1, -.5, 0)
        glTexCoord2i(1, 0)
        glVertex3f(-1, -.5, 0)
        glTexCoord2i(1, 1)
        glVertex3f(-1, -.5, 1.5)
        glTexCoord2i(0, 1)
        glVertex3f(1, -.5, 1.5)
        glEnd()
        glPopMatrix()
        glBindTexture(Key.texture.target, 0)
        glDisable(GL_BLEND)
        glEnable(GL_DEPTH_TEST)

    # Draw 2D
    glViewport(0, 0, window.width, window.height)
    glMatrixMode(gl.GL_PROJECTION)
    glLoadIdentity()
    glOrtho(0, window.width, 0, window.height, -1, 1)
    glMatrixMode(gl.GL_MODELVIEW)
    glLoadIdentity()
    
    # Fade
    if game.blink > 0:
        glDisable(GL_DEPTH_TEST)
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glColor4f(0, 0, 0, game.blink)
        glBegin(GL_QUADS)
        glVertex2f(0, 0)
        glVertex2f(window.width, 0)
        glVertex2f(window.width, window.height)
        glVertex2f(0, window.height)
        glEnd()
        glDisable(GL_BLEND)
        glEnable(GL_DEPTH_TEST)

    if game.player.keys > 0:
        keys_sprite.draw()
        keys_display.draw()
    if game.level.stars_max > 0:
        stars_sprite.draw()
        stars_display.draw()

@window.event
def on_key_press(symbol, modifiers):
    key = pyglet.window.key
    
    global isPlaying
    if not isPlaying:
        if symbol == key.ESCAPE:
            window.close()
        elif symbol == key.UP or symbol == key.LEFT or symbol == key.RIGHT or \
				symbol == key.W or symbol == key.A or symbol == key.D:
            isPlaying = True
        return pyglet.event.EVENT_HANDLED

    if symbol == key.ESCAPE:
        isPlaying = False
        return pyglet.event.EVENT_HANDLED
    elif symbol == key.W or symbol == key.UP:
        game.player.moveForward()
    elif symbol == key.A or symbol == key.LEFT:
        game.player.turnLeft()
    elif symbol == key.D or symbol == key.RIGHT:
        game.player.turnRight()

@window.event
def on_resize(width, height):
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(90, window.width / window.height, .1, 100)
    glMatrixMode(GL_MODELVIEW)
    return pyglet.event.EVENT_HANDLED

def update(dt):
    global isPlaying
    if not isPlaying:
        return
    
    if joysticks:
        if joysticks[0].y < -0.9:
            game.player.moveForward()
        elif joysticks[0].x < -0.9:
            game.player.turnLeft()
        elif joysticks[0].x > 0.9:
            game.player.turnRight()
        elif joysticks[0].buttons[7]:
            isPlaying = False
    
    game.update(dt)
    game.player.update(dt)
    
    for entity in list(game.entities):
        if math.fabs(entity.x - game.player.x) < 0.1 and \
                math.fabs(entity.y - game.player.y) < 0.1:
            destroy = entity.pickup()
            if destroy:
                game.entities.remove(entity)
    for entity in game.entities:
        entity.update(dt)
    
    keys_display.text = str(game.player.keys)
    stars_display.text = str(game.player.stars) + "/" + \
            str(game.level.stars_max)

pyglet.clock.schedule_interval(update, 1 / 60.0)

pyglet.app.run()
